
.include "msp430g2x31.inc"

LED0_LEN equ 0x200
LED1_LEN equ 0x202
LED2_LEN equ 0x204
LED3_LEN equ 0x206
LED4_LEN equ 0x208
LED5_LEN equ 0x20a
LED6_LEN equ 0x20c
LED7_LEN equ 0x20e
LED8_LEN equ 0x210
LED9_LEN equ 0x212

LED0_DIR equ 0x220
LED1_DIR equ 0x222
LED2_DIR equ 0x224
LED3_DIR equ 0x226
LED4_DIR equ 0x228
LED5_DIR equ 0x22a
LED6_DIR equ 0x22c
LED7_DIR equ 0x22e
LED8_DIR equ 0x230

  org 0xf800
start:
  ;; Turn off watchdog
  mov.w #0x5a80, &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set up crystal
  mov.b #(DCO_3), &DCOCTL
  mov.b #(RSEL_15), &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  mov.b #0xff, &P1DIR        ; P1.0 P1.1
  mov.b #0x00, &P1OUT
  mov.b #0x80, &P2DIR        ; P1.0 P1.1
  mov.b #0x00, &P2OUT
  mov.b #0x00, &P2SEL
  mov.b #0x40, &P2REN

  ;; Set up A/D
  ;mov.w #(ADC10ON), &ADC10CTL0
  ;mov.w #(INCH_2|ADC10SSEL_2), &ADC10CTL1

  ;; Set up Timer
  mov.w #3000, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  mov.w #900, &LED0_LEN
  mov.w #1000, &LED1_LEN
  mov.w #800, &LED2_LEN
  mov.w #900, &LED3_LEN
  mov.w #700, &LED4_LEN
  mov.w #600, &LED5_LEN
  mov.w #500, &LED6_LEN
  mov.w #800, &LED7_LEN
  mov.w #700, &LED8_LEN

  mov.w #1, &LED0_DIR
  mov.w #-1, &LED1_DIR
  mov.w #1, &LED2_DIR
  mov.w #-1, &LED3_DIR
  mov.w #1, &LED4_DIR
  mov.w #-1, &LED5_DIR
  mov.w #1, &LED6_DIR
  mov.w #-1, &LED7_DIR
  mov.w #1, &LED8_DIR

  ;; Okay, I can be interrupted now
  eint

  mov.w #1, r10    ; count direction
  mov.w #1, r12    ; update flag
  mov.w #1, r7     ; night count (1 through 8)

main:
  ;cmp.w #5, r12
  ;jeq main
  ;cmp.w #0xff, &P1OUT
  ;jeq main

  bit.b #0x40, &P2IN
  jeq skip_update_night
  mov.b #0x00, &P1OUT
  mov.b #0x00, &P2OUT
wait_debounce:
  mov.w #0xffff, r8
wait_debounce_delay:
  dec.w r8
  jnz wait_debounce_delay
  bit.b #0x40, &P2IN
  jnz wait_debounce
  inc.w r7
  cmp.w #9, r7
  jne skip_update_night
  mov.w #1, r7
skip_update_night:

update_led8:
  cmp.w &LED8_LEN, &TAR
  jl update_led0
  bis.b #0x80, &P2OUT

update_led0:
  cmp.w &LED0_LEN, &TAR
  jl update_led1
  bis.b #0x01, &P1OUT

update_led1:
  cmp.w #1, r7
  jeq main
  cmp.w &LED1_LEN, &TAR
  jl update_led2
  bis.b #0x02, &P1OUT

update_led2:
  cmp.w #2, r7
  jeq main
  cmp.w &LED2_LEN, &TAR
  jl update_led3
  bis.b #0x04, &P1OUT

update_led3:
  cmp.w #3, r7
  jeq main
  cmp.w &LED3_LEN, &TAR
  jl update_led4
  bis.b #0x08, &P1OUT

update_led4:
  cmp.w #4, r7
  jeq main
  cmp.w &LED4_LEN, &TAR
  jl update_led5
  bis.b #0x10, &P1OUT

update_led5:
  cmp.w #5, r7
  jeq main
  cmp.w &LED5_LEN, &TAR
  jl update_led6
  bis.b #0x20, &P1OUT

update_led6:
  cmp.w #6, r7
  jeq main
  cmp.w &LED6_LEN, &TAR
  jl update_led7
  bis.b #0x40, &P1OUT

update_led7:
  cmp.w #7, r7
  jeq main
  cmp.w &LED7_LEN, &TAR
  jl main
  bis.b #0x80, &P1OUT

  jmp main

timer_interrupt:
  mov.b #0x00, &P1OUT
  mov.b #0x00, &P2OUT
  ;mov.w #0, r12

.if 0
  add.w r10, &LED0_LEN
  cmp.w #1093, &LED0_LEN
  jne not_top
  mov.w #-1, r10
not_top:
  cmp.w #0, &LED0_LEN
  jne not_bottom
  mov.w #1, r10
not_bottom:
.endif

  mov.w #LED0_LEN, r5
  mov.w #LED0_DIR, r6
inc_loop:
  add.w @r6, 0(r5)
  cmp.w #2000, 0(r5)
  jne not_top
  mov.w #-1, 0(r6)
not_top:
  cmp.w #0, 0(r5)
  jne not_bottom
  mov.w #1, 0(r6)
not_bottom:
  add.w #2, r5
  add.w #2, r6
  cmp.w #LED9_LEN, r5
  jne inc_loop

exit_interrupt:
  reti

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset


